package sudoku;

import org.junit.jupiter.api.Disabled;
import org.junit.jupiter.api.Test;
import static org.junit.jupiter.api.Assertions.*;
import java.util.stream.IntStream;

class SolverTest {
    @Test
    void constructorTest() {
        Solver solver = new Solver();
        assertNotNull(solver);
    }

    @Test
    void setBoardTest() {
        Solver solver = new Solver();
        int[][] board = new int[9][9];
        solver.setBoard(board);
        assertEquals(board, solver.getBoard());
    }

    @Test
    void randomizeBoardTest() {
        int[][] board = new int[9][9];
        Solver solver = new Solver();
        solver.randomizeBoard();
        assertNotEquals(board, solver.getBoard());
    }

    @Test
    void legalTest() {
        Solver solver = new Solver();
        assertTrue(solver.isLegal(0, 0, 1));
        solver.set(0, 0, 1);

        IntStream.range(0, 9).forEach(i -> {
            assertFalse(solver.isLegal(0, i, 1));
            assertFalse(solver.isLegal(i, 0, 1));
        });

        assertTrue(solver.isLegal(5, 5, 1));
        assertTrue(solver.isLegal(8, 8, 9));
        assertTrue(solver.isLegal(8, 8, 1));
    }

    @Test
    void solverTest() {
        Solver solver = new Solver();
        assertFalse(solver.isSolved());
        assertTrue(solver.solve());
        assertTrue(solver.isSolved());

        solver.clear();
        solver.randomizeBoard();
        assertFalse(solver.isSolved());
        assertTrue(solver.solve());
        solver.clear();
        solver.randomizeBoard();
        assertFalse(solver.isSolved());
    }

    @Test
    void resetTest() {
        Solver solver = new Solver();
        assertTrue(solver.solve());
        solver.randomizeBoard();
        solver.clear();
        assertFalse(solver.isSolved());
    }

    @Test
    void legalWithInvalidInputsTest() {
        Solver solver = new Solver();
        assertFalse(solver.isLegal(-1, 0, 1));
        assertFalse(solver.isLegal(0, -1, 1));
        assertFalse(solver.isLegal(0, 0, -1));
        assertFalse(solver.isLegal(0, 0, 10));
    }

    @Test
    @Disabled
    void unsolvableTest() {
        Solver solver = new Solver();
        solver.set(0, 0, 1);
        solver.set(0, 1, 1);
        assertFalse(solver.solve());
    }
}
